library(magrittr)
library(data.table)
library(ggplot2)
library(gridExtra)
library(here)
library(hmod)

dir = here()
text.size = 10

# world bank gini
setwd(dir)
setwd("../data/world_bank")
wdi = fread("wdi.csv") %>% na.omit()
wdi$SI.POV.GINI = wdi$SI.POV.GINI / 100

  # max ineq
  id =  which(wdi$SI.POV.GINI == max(wdi$SI.POV.GINI))
  max_lab = paste(wdi$country[id], wdi$year[id], sep = ", ")

  # max ineq
  id =  which(wdi$SI.POV.GINI == min(wdi$SI.POV.GINI))
  min_lab = paste(wdi$country[id], wdi$year[id], sep = ", ")

  
#-------------------------------------------------------
# model
firm_size = 10^5
s = 3
beta_vec = seq(0.1, 1.1, length.out = 100)
g = rep(NA, length(firm_size))

for(i in 1:length(beta_vec)){
  
  h = hierarchy(firm_size, s)
  hp = hierarchical_power(h)
  
  pay = hp^beta_vec[i]
  dist = rep(pay, h)
  g[i] = gini(dist)
  
}

model = data.table(beta = beta_vec, g)

# plot
mod_plot = ggplot() + 
  geom_hline(yintercept = max(wdi$SI.POV.GINI), col = "red", linetype = 2) +
  geom_hline(yintercept = min(wdi$SI.POV.GINI), col = "red", linetype = 2) +
  geom_line(data = model, aes(x = beta, y = g), size = 1, col = "dodgerblue4") +
  scale_x_continuous("Degree of hierarchical despotism (D)", breaks = seq(0, 1.1, 0.1)) +
  scale_y_continuous("Income inequality (Gini index)", breaks = seq(0, 1.1, 0.1)) +
  theme_bw() +
  theme(panel.border = element_rect(colour = "black"),
        panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(), 
        legend.position = c(0.2, 0.85),
        legend.title = element_blank(),
        legend.text = element_text(size = rel(0.8)),
        plot.title = element_text(face="bold", hjust = 0.5, size = rel(1)),
        axis.line = element_line(color = "black"), 
        axis.title.x=element_text(vjust=-0.3, size=rel(0.9)),
        axis.title.y=element_text(vjust= 1.1, size=rel(0.9)),
        axis.text.x = element_text(margin=margin(5,5,0,0,"pt")),
        axis.text.y = element_text(margin=margin(3,5,0,3,"pt")),
        axis.ticks.length = unit(-0.7, "mm"),
        text=element_text(size = text.size, family="Times")) +
  annotate("text", label = "A hierarchy with\n100,000 members\n(span of control = 3)", 
           x = 0.78, y = 0.45, size = 3, family = "Times") +
  annotate("text", label = max_lab, x = 0.3, y = 0.68, size = 2.5, family = "Times") +
  annotate("text", label = min_lab, x = 0.8, y = 0.18, size = 2.5, family = "Times")


# export
setwd(dir)
gA = ggplotGrob(mod_plot)

png("beta_effect.png", width = 5, height = 4.5, res = 600, units = "in")
grid.arrange(gA)
dev.off()
