library(hmod)
library(magrittr)
library(data.table)
library(ggplot2)
library(gridExtra)
library(here)

text.size = 10
dir = here()

# energy firm size regression
#--------------------------------------------------
setwd(dir)
setwd("../data/gem")
energy = fread("energy_v_firm.csv")

e_predict = function(firm_size){
  
  x = log(energy$energy)
  y = log(energy$firm_mean)
  r = lm(y ~ x)
  
  a = exp(coef(r)[1])
  b = coef(r)[2]

    y = (firm_size/a)^(1/b)
  return(y)
}

# model
#--------------------------------------------------
s = span_empirical
span = boot_span(s$rank, s$span)
a = span[1]
b = span[2]
sigma = 0

#  low energy
#--------------------------------------------------

# simulation
n = 5e4
alpha = 3.5
large_firm_sample = rpld(10^7, 1, alpha, 2.3*10^4)  
low_mean_firm = mean(large_firm_sample)
low_energy = e_predict(low_mean_firm) %>% round()

firm_sim = sample(large_firm_sample, 10^6)
base_sim = base_fit(a, b, firm_sim)

r_sim = rep(1.1, n)
base_pay_sim = rep(1, n)

mod_low = model_r(a, b, base_sim, firm_sim, base_pay_sim, r_sim, sigma, power = T) %>% data.table()
low_power_gini = gini(mod_low$V2) %>% round(.,2)

low_energy_label = paste(  "Energy use per capita ~ ",
                           low_energy, "GJ per person",
                           "\nConcentration of hierarchical power: Gini = ", low_power_gini)


# firm grid
#--------------------------------------------------
firm_sim = sample(large_firm_sample, n)
base_sim = base_fit(a, b, firm_sim)

coord = firm_grid(base_sim, 40)
grid = grid_plot_r(a, b, sigma, coord, base_sim, base_pay_sim, r_sim)
colnames(grid) <- c("Firm_ID", "x_", "y_", "z_", "pay")

# camera parameters
e = c(-4, -4, -4)
c1 = c(-300, -300, 400)
theta = c(pi/2, 0, -pi/2.2)

# 2D projection
r = ((grid[,2]-c1[1])^2 + (grid[,3] - c1[2])^2)^0.5
grid = grid[order(-r),]

# cut off grid
grid = grid[grid[,2] <= 300, ] # y cutoff
grid = grid[grid[,3] <= 500, ] # x cutoff

p.bottom = cbind(grid[, 2:3], z_ =  exp( 0.4*(grid[,4] - 1) ) - 1 )
bottom = project(p.bottom, c1, theta, e)  
colnames(bottom) <- c("x", "y")

grid[,4] =  exp( 0.4*grid[,4])
top = project(grid[, 2:4], c1, theta, e) 
colnames(top) <- c("x", "y")
people.coord = data.table(bottom = bottom, 
                          top = top, 
                          rank =  grid[,4])

x_max = max(grid[,2])
y_max = max(grid[,3])

border = rbind(
  c(-1, -1, 0),
  c(x_max + 1, 0, 0),
  c(x_max + 1, y_max + 1, 0),
  c(-1, y_max + 1, 1)
)

border.coord = project(border, c1, theta, e) %>% data.table()

# plot
g_low = ggplot() +
  geom_polygon(data = border.coord, aes(x = V1, y = V2), fill = NA, col = "grey60", size = 0.1) + 
  geom_segment(data = people.coord,
               aes(x = bottom.x, y = bottom.y, xend  = top.x, yend = top.y, col = rank), size = 0.2) +
  scale_color_gradientn("Hierarchical Rank\n",    colours = rainbow(4)) +
  coord_cartesian(xlim = c(min(people.coord$bottom.x), max(people.coord$bottom.x))*0.95, 
                  ylim = c(-1, 2.5)) + 
  theme_bw() +
  theme(panel.border = element_blank(),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        legend.position = "none",
        legend.key.width = unit(0.3, "cm"),
        legend.text = element_text(size = rel(0.8)),
        legend.title = element_text(size = rel(0.8), vjust = 0),
        text=element_text(size= text.size, family="Times"),
        axis.title = element_blank(),
        axis.ticks = element_blank(),
        axis.text = element_blank()
  ) + 
  annotate("text", label = "Subsistence Society", 
           x = mean(range(people.coord$bottom.x)) , y = 2.5, family = "Times", size = 4.5, fontface = 2 ) +
  annotate("text", label = low_energy_label, 
           x = mean(range(people.coord$bottom.x)) , y = 2, family = "Times", size = 3.5 )






# high energy
#--------------------------------------------------
# simulation

n = 30000
alpha = 1.95
large_firm_sample = rpld(10^7, 1, alpha, 2.3*10^4, 2.3*10^6)  
high_mean_firm = mean(large_firm_sample)

high_energy = e_predict(high_mean_firm) 
high_energy = round(high_energy/50) * 50

firm_sim = sample(large_firm_sample, 10^6)
base_sim = base_fit(a, b, firm_sim)

mod_high = model_r(a, b, base_sim, firm_sim, base_pay_sim, r_sim, sigma, power = T) %>% data.table()
high_power_gini = gini(mod_high$V2) %>% round(.,2)

high_energy_label = paste(  "Energy use per capita ~ ",
                            high_energy, "GJ per person",
                            "\nConcentration of hierarchical power: Gini = ", high_power_gini)


# firm grid
#--------------------------------------------------
firm_sim = sample(large_firm_sample, n)
base_sim = base_fit(a, b, firm_sim)

coord = firm_grid(base_sim, 10)
grid = grid_plot_r(a, b, sigma, coord, base_sim, base_pay_sim, r_sim)
colnames(grid) <- c("Firm_ID", "x_", "y_", "z_", "pay")

# camera parameters
e = c(-4, -4, -4)
c1 = c(-300, -300, 400)
theta = c(pi/2, 0, -pi/2.2)

# 2D projection
r = ((grid[,2]-c1[1])^2 + (grid[,3] - c1[2])^2)^0.5
grid = grid[order(-r),]

# cut off grid
grid = grid[grid[,2] <= 300, ] # y cutoff
grid = grid[grid[,3] <= 500, ] # x cutoff

p.bottom = cbind(grid[, 2:3], z_ =  exp( 0.4*(grid[,4] - 1) ) - 1 )
bottom = project(p.bottom, c1, theta, e)  
colnames(bottom) <- c("x", "y")

grid[,4] =  exp( 0.4*grid[,4])
top = project(grid[, 2:4], c1, theta, e) 
colnames(top) <- c("x", "y")

people.coord = data.table(bottom = bottom, top = top, rank =  grid[,4])

x_max = max(grid[,2])
y_max = max(grid[,3])

border = rbind(
  c(-1, -1, 0),
  c(x_max + 1, 0, 0),
  c(x_max + 1, y_max + 1, 0),
  c(-1, y_max + 1, 1)
)

border.coord = project(border, c1, theta, e) %>% data.table()

# plot
g_high = ggplot() +
  geom_polygon(data = border.coord, aes(x = V1, y = V2), fill = NA, col = "grey60", size = 0.1) + 
  geom_segment(data = people.coord,
               aes(x = bottom.x, y = bottom.y, xend  = top.x, yend = top.y, col = (top.y - bottom.y)), size = 0.2) +
  scale_color_gradientn("Relative Income\n(median = 1)\n",  colours = rainbow(4)) +
  coord_cartesian(xlim = c(min(people.coord$bottom.x), max(people.coord$bottom.x))*0.95, 
                  ylim = c(-1, 2.5)) + 
  theme_bw() +
  theme(panel.border = element_blank(),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        legend.position = "none",
        legend.key.width = unit(0.3, "cm"),
        legend.text = element_text(size = rel(0.8)),
        legend.title = element_text(size = rel(0.8), vjust = 0),
        text=element_text(size= text.size, family="Times"),
        axis.title = element_blank(),
        axis.ticks = element_blank(),
        axis.text = element_blank()
  ) +
  annotate("text", label = "Industrial Society", 
           x = mean(range(people.coord$bottom.x)) , y = 2.6, family = "Times", size = 4.5, fontface = 2 ) +
  annotate("text", label = high_energy_label, 
           x = mean(range(people.coord$bottom.x)) , y = 2.1, family = "Times", size = 3.5 )


# export
#--------------------------------------------------
setwd(dir)

gA = ggplotGrob(g_low)
gB = ggplotGrob(g_high)

png("mod_landscape.png", width = 8, height = 8, units = 'in', res = 600)
  grid.arrange(gA, gB, nrow = 2)
dev.off()

