library(magrittr)
library(data.table)
library(ggplot2)
library(gridExtra)
library(here)
library(hmod)

dir = here()
text.size = 10

#-------------------------------------------------------
# effect of hierarchy size on hierarchical power
span = 3
size = 10^(seq(0, 7, length.out = 500)) %>% round() %>% unique()
g = rep(NA, length(size))
ceo_ratio = rep(NA, length(size))

for(i in 1:length(size)){

  h = hierarchy(size[i], span)
  hp = hierarchical_power(h)
  dist = rep(hp, h)

  # gini index
  g[i] = gini(dist)

  # ceo ratio
  ceo_ratio[i] = max(dist) / mean(dist)

}

mod = data.table(size, g, ceo_ratio)

# gini index plot
bks = 10^(0:7)
lbs = prettyNum(bks, big.mark = ",", scientific = F)

gini_plot = ggplot(data = mod) +
  geom_path(aes(x = size, y = g), size = 1, col = "dodgerblue4") +
  scale_x_log10("Hierarchy size (number of members)",
                breaks = bks, labels = lbs) +
  scale_y_continuous("Concentration of hierarchical power (Gini index)", breaks = seq(0, 1, 0.1)) +
  theme_bw() +
  theme(panel.border = element_rect(colour = "black"),
        panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(), 
        legend.position = c(0.2, 0.85),
        legend.title = element_blank(),
        legend.text = element_text(size = rel(0.8)),
        plot.title = element_text(face="bold", hjust = 0.5, size = rel(1)),
        axis.line = element_line(color = "black"), 
        axis.title.x = element_text(vjust=-0.3, size=rel(0.9)),
        axis.title.y = element_text(vjust= 1.1, size=rel(0.9)),
        axis.text.x = element_text(margin=margin(5,5,0,0,"pt")),
        axis.text.y = element_text(margin=margin(3,5,0,3,"pt")),
        axis.ticks.length = unit(-0.7, "mm"),
        text=element_text(size = text.size, family="Times")) +
  annotate("text", label = "A hierarchy with\nspan of control = 3",
           x = 30, y = 0.8, size = 3, family = "Times")



# pay ratio plot
bks = 10^(seq(0, 7, 2))
lbs = prettyNum(bks, big.mark = ",", scientific = F)

ratio_plot = ggplot(data = mod) +
  geom_path(aes(x = size, y = ceo_ratio), col = "red") +
  scale_x_log10("Hierarchy size (number of members)",
                breaks = bks, labels = lbs) +
  scale_y_log10("Ratio", breaks = bks, labels = lbs) +
  ggtitle("Hierarchical power of the ruler\nrelative to the average") +
  theme_bw() +
  theme(panel.border = element_rect(colour = "black"),
        panel.grid.major = element_blank(), 
        panel.grid.minor = element_blank(), 
        legend.position = c(0.2, 0.85),
        legend.title = element_blank(),
        legend.text = element_text(size = rel(0.8)),
        plot.title = element_text(face="bold", hjust = 0.5, size = rel(1)),
        axis.line = element_line(color = "black"), 
        axis.title.x = element_text(vjust=-0.3, size=rel(0.9)),
        axis.title.y = element_text(vjust= 1.1, size=rel(0.9)),
        axis.text.x = element_text(margin=margin(5,5,0,0,"pt")),
        axis.text.y = element_text(margin=margin(3,5,0,3,"pt")),
        axis.ticks.length = unit(-0.7, "mm"),
        text=element_text(size = text.size*0.7, family="Times"))

# inset
inset = ggplotGrob(ratio_plot)
gini_plot = gini_plot + 
  annotation_custom(inset, xmin = 2.3, xmax = 7.3, ymin = 0.0, ymax = 0.55)

# export
setwd(dir)
gA = ggplotGrob(gini_plot)

png("size_effect.png", width = 5, height = 4.6, res = 600, units = "in")
grid.arrange(gA)
dev.off()

